@@ -15,7 +15,7 @@
 import pretend
 import pytest
 
-from sigstore._verify import verify
+from sigstore._verify import VerificationFailure, VerificationSuccess, verify
 
 
 @pytest.mark.xfail
@@ -23,4 +23,9 @@ def test_verify():
     filename = pretend.stub()
     certificate_path = pretend.stub()
     signature_path = pretend.stub()
-    assert verify(filename, certificate_path, signature_path) == "Nothing here yet"
+    assert verify(filename, certificate_path, signature_path) is not None
+
+
+def test_verify_result_boolish():
+    assert not VerificationFailure(reason="foo")
+    assert VerificationSuccess()
